/* sieve_graphics.cs is to visualize the distribution of primes */
/* calculated in sieve4.cs into mk[]. Data on datavault. */  
 
#include <stdio.h>
#include <stdlib.h>
#include <cm/cmfs.h>
#include <cm/cm_file.h>

/* The following lines are necessary to include the display.h */
/* header file because of various bugs. */

#define current Current
#define volatile
#include <cm/display.h>
#undef volatile
#undef current

#define RSIZE 16384  
#define CSIZE 32768  
#define SSIZE 1024
#define NGRP 512
#define NBLK 32  

shape [CSIZE]sieve;
shape [SSIZE][SSIZE]screen;

bool:sieve mk[RSIZE];  
char:screen image;
int min,max;

void distr_graph()
 { int:sieve x,y,np,cm;
   unsigned char:sieve color;
   int cg,cb,bm;
   double a,b; 
   char cont,display;

   if (min == 0) min++;  
   a = 254.0/(max-min);
   b = 1.0 - a * (min - 1.0); 
   bm = 0;

   printf("a = %10.5f\n",a);
   printf("b = %10.5f\n",b);
 
   with(sieve)
   { cm = pcoord(0);  
     y = cm/NBLK;
     x = NBLK * cm %% 1024;

     CMSR_select_display_menu(8,SSIZE,SSIZE);

     CMSR_display_set_color_map("rainbow");

     for(cb=0; cb < NBLK; cb++)
      { np = 0;
        for(cg=0; cg < NGRP; cg++)
         { np = np + mk[bm];
           bm++;  
         } /* end for */  

        where (np == 0)
          color = 0;
        else color = (a * (np - 1) ) + b;

        printf("color = %d\n",[0]color);

        [x][y]image = color;
        x++;   
      } /* end for */
    } /* end with(sieve) */ 

     with(screen) {      
     printf("Hit a key to see display>");
     scanf("%1s",&display); 

     CMSR_write_to_display(&image);

     printf("Hit a key to continue>");
     scanf("%1s",&cont);  

    } /* end with(screen) */
}

void find_min_max()
{ int:sieve np,mn,mx;
  int rm,bs,gs;

  rm = 0;
  min = max = 0;
 
  with (sieve)
   { mx = 0;
     mn = NGRP+1;
   
     for(bs=0; bs < NBLK; bs++)
      {  np = 0;
         for(gs=0; gs < NGRP; gs++)
          { np += mk[rm];
            rm++;  
          } /* end for */   
   
         mn <?= np;    
         mx >?= np;    
   
       } /* end for */  
     min = (<?= mn);
     max = (>?= mx); 
  } /* end with */

printf("min = %d\n",min);
printf("max = %d\n",max);
}

void checkpoint()
 {
   int mark,ispr,dvfd;
   char fnm[50], type[2];

   ispr = boolsizeof(bool:sieve);
   mark = RSIZE * ispr;

   printf("r,w> ");  scanf("%s",type);
   printf("Checkpoint file> ");  scanf("%s", fnm);

   with (sieve)
   { if ( ( type[0] == 'w' ) && ( type[1] == 0 ) )
      { /* Checkpoint in user-specified files */
        CM_start_timer(1);
        dvfd = CMFS_creat(fnm, 0600);
        CMFS_write_file(dvfd, &mk[0], mark);
        CMFS_close(dvfd);
        CM_stop_timer(1);  CM_reset_timer();
      }
    else if ( ( type[0] == 'r' ) && ( type[1] == 0 ) )
      { /* Resume from user-specified files */
        CM_start_timer(1);
        dvfd = CMFS_open(fnm, CMFS_O_RDONLY);
        CMFS_read_file(dvfd, &mk[0], mark);
        CMFS_close(dvfd);
        CM_stop_timer(1);  CM_reset_timer();
      }
   }
}

/* Main program. Commands are:
   x  - checkpoint
   r  - run distr_graph     
   f  - run find_min_max 
   q  - quit               */ 
 
void main()
{ char c[2];
  
  while ( printf("sieve_graphics>"), scanf("%1s", c), c[0] != 'q' )
   switch (c[0])
    { case 'x':        
        checkpoint();
        break;
      case 'f':
        find_min_max();
        break;   
      case 'r': 
        CM_start_timer(1); 
        distr_graph(); 
        CM_stop_timer(1);  CM_reset_timer(); 
        break;
      default:   printf("Bad input - ignored\n"); break;
    }
} 
  
